@@ -4,7 +4,6 @@ from asyncio import CancelledError
 from typing import TYPE_CHECKING, Any, Final, List, Optional, Union
 
 from robotcode.core.async_tools import async_tasking_event, threaded
-from robotcode.core.logging import LoggingDescriptor
 from robotcode.core.lsp.types import (
     DeclarationParams,
     Location,
@@ -13,6 +12,7 @@ from robotcode.core.lsp.types import (
     ServerCapabilities,
     TextDocumentIdentifier,
 )
+from robotcode.core.utils.logging import LoggingDescriptor
 from robotcode.jsonrpc2.protocol import rpc_method
 from robotcode.language_server.common.decorators import language_id_filter
 from robotcode.language_server.common.has_extend_capabilities import (
