@@ -4,8 +4,8 @@ import ast
 from typing import TYPE_CHECKING, Any, List, Optional
 
 from robot.parsing.model.blocks import If, Keyword, TestCase
-from robotcode.core.logging import LoggingDescriptor
 from robotcode.core.lsp.types import FoldingRange
+from robotcode.core.utils.logging import LoggingDescriptor
 
 from ...common.decorators import language_id
 from ...common.text_document import TextDocument
